<?php declare(strict_types=1);

namespace Drupal\transcoding_aws\Controller;

use Aws\Sns\Message;
use Aws\Sns\MessageValidator;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\transcoding\TranscodingStatus;
use Drupal\transcoding_aws\Exception\IgnoredMessageException;
use GuzzleHttp\Client;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

/**
 * Handle SNS postbacks.
 */
class SNS extends ControllerBase {

  /**
   * Message type.
   *
   * @var string
   */
  protected $messageType;

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory')
    );
  }

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Config factory.
   */
  public function __construct(ConfigFactoryInterface $configFactory) {
    $this->configFactory = $configFactory;
  }

  /**
   * Handle the posted data/page callback.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   */
  public function handlePost() {
    try {
      $message = Message::fromRawPostData();
      $data = $this->validateMessage($message);
    }
    catch (IgnoredMessageException $e) {
      return new Response();
    }
    catch (\Exception $e) {
      throw new AccessDeniedHttpException();
    }
    switch ($this->messageType) {
      case 'SubscriptionConfirmation':
      case 'UnsubscribeConfirmation':
        $this->handleSubscriptionChange($message);
        break;
      case 'Notification':
        $this->processNotification($data);
        break;
      default:
        break;
    }
    return new Response();
  }

  /**
   * Validate the message.
   *
   * @param \Aws\Sns\Message $message
   *
   * @return array
   *   Message array.
   *
   * @throws \Drupal\transcoding_aws\Exception\IgnoredMessageException
   */
  protected function validateMessage(Message $message): array {
    if (!(new MessageValidator())->isValid($message) || empty($message->toArray()['Type'])) {
      throw new \Exception();
    }
    $messageData = $message->toArray();
    $this->messageType = $messageData['Type'];
    if ($this->messageType !== 'Notification') {
      return [];
    }
    try {
      $data = \GuzzleHttp\json_decode($messageData['Message'], TRUE);
    }
    catch (\InvalidArgumentException $e) {
      throw new BadRequestHttpException();
    }
    $siteUuid = $this->configFactory->get('system.site')->get('uuid');
    if (!empty($data['userMetadata']['drupal-site-uuid'])
      && $data['userMetadata']['drupal-site-uuid'] != $siteUuid
    ) {
      throw new IgnoredMessageException('Notification not for our site.');
    }
    return $data;
  }

  /**
   * Handle subscription changes by pinging their callback.
   *
   * @param \Aws\Sns\Message $message
   */
  protected function handleSubscriptionChange(Message $message) {
    $client = new Client();
    $client->get($message->toArray()['SubscribeURL']);
  }

  /**
   * Process a notification.
   *
   * @param array $data
   *   Message data.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function processNotification(array $data): void {
    $job = $this->entityTypeManager()->getStorage('transcoding_job')
      ->loadByProperties(['remote_id' => $data['jobId']]);
    if ($job) {
      /** @var \Drupal\transcoding\TranscodingJobInterface $job */
      $job = reset($job);
    }
    else {
      throw new BadRequestHttpException();
    }
    switch ($data['state']) {
      case 'ERROR':
        $job->set('status', TranscodingStatus::FAILED);
        break;
      case 'COMPLETED':
        $job->set('status', TranscodingStatus::PROCESSED);
        break;
      default:
        throw new BadRequestHttpException(
          sprintf('Unrecognized state %s', $data['state'])
        );
    }
    $job->setServiceData([
      'result' => $data,
    ] + $job->getServiceData());
    $job->save();
  }

}
